#include "MarkerGroup.h"
#include "Error.h"
#include "StringArray.h"
#include <iostream>

GroupInfo::GroupInfo()
{
 start=0;
 end = 0;
 count=0;
}

GroupInfo::~GroupInfo(){}

MarkerGroup::MarkerGroup()
{
 min_freq = 1.0;
 count = 0;
}

MarkerGroup::~MarkerGroup(){}

void MarkerGroup::LoadMarkerGroupInfo(std::string marker_info_file)
{
 FILE *fh = fopen(marker_info_file.c_str(), "r");
 if(fh==NULL) error("Open marker info %s failed!\n", marker_info_file.c_str() );

 std::string SET("GROUP");
 std::string CHR("CHR");
 std::string POS("POS");
 std::string WEIGHT("WEIGHT");
 std::string FREQ("FREQ");

 String buffer;
 StringArray tokens;
 
 while(!feof(fh) )
 {
  buffer.ReadLine(fh);
  if(buffer.Length()==0) continue;

  tokens.ReplaceTokens(buffer);
  if(tokens.Length()<3) error("Too few columns in the marker info file!\n");

  if(tokens[0][0]=='#')
  {
   tokens[0] = tokens[0].SubStr(1, tokens[0].Length()-1);

   for(int i=0; i<tokens.Length(); i++)
    index[std::string(tokens[i].ToUpper().c_str())] = i;

  if(index.find(SET)==index.end() || index.find(CHR)==index.end() || index.find(POS)==index.end() )
	error("CHR or POS or GROUP is missing in the header!\n");
   continue;
  }

  std::string group_name = std::string(tokens[index[SET]].c_str() );
  std::string chr_name = std::string(tokens[index[CHR]].c_str() );

  if(index.find(FREQ)!=index.end())
  {
    double freq = tokens[index[FREQ]].AsDouble();
    if(freq > 0.5) freq = 1-freq;
    if( freq > min_freq) continue;
  }

  if(groups.find(group_name)==groups.end() ) group_names.push_back(group_name);
  groups[group_name].chr.push_back(chr_name);
  groups[group_name].pos.push_back(tokens[index[POS]].AsInteger());
  if(index.find(WEIGHT)!=index.end()) groups[group_name].weight.push_back(tokens[index[WEIGHT]].AsDouble());
  if(index.find(FREQ)!=index.end())   groups[group_name].freq.push_back(tokens[index[FREQ]].AsDouble());

  groups[group_name].count++;
 }

 for(int i=0; i<group_names.size(); i++)
 {
  groups[group_names[i]].start = groups[group_names[i]].pos[0];
  groups[group_names[i]].end = groups[group_names[i]].pos[groups[group_names[i]].count-1];
 }
  count = group_names.size();
}

void MarkerGroup::PrintGroupInfo()
{
 for(int i=0; i<group_names.size(); i++)
 {
  cout<<group_names[i]<<" : "<<endl;
  for(int j=0; j<groups[group_names[i]].count; j++)
   cout<<groups[group_names[i]].chr[j]<< " " <<groups[group_names[i]].pos[j]<< " " << groups[group_names[i]].weight[j]<<endl;
 }
}
