#include "PedDataAnalyzer.h"
#include <iostream>

#define NROUNDS 100

PedDataAnalyzer::PedDataAnalyzer()
{
 pheno = NULL;
 mg = NULL;
 min_freq = 1.0;
 toEstimateFreq = false;
}

PedDataAnalyzer::~PedDataAnalyzer(){}

void PedDataAnalyzer::LoadAndAnalyze(std::string file, std::string outfile)
{
 VCFInputFile vin(file.c_str());
 if(pheno->pids.size()==0) pheno->GetPIDs();
 vin.includePeople(pheno->pids);

// Set flags to indicate the availability of genotype data for all individauls in the pedfile
 std::vector<bool> isGenotyped;
 isGenotyped.resize(pheno->nIndividuals);
 VCFPeople &people = vin.record.getPeople();
 for(int p=0; p<people.size(); p++)
 {
  if(pheno->pids_map.find( (*people[p]).getName()) == pheno->pids_map.end() ) continue;
  isGenotyped[pheno->pids_map[(*people[p]).getName()]] = true;
 }

// Output file
 FILE *fh = fopen(outfile.c_str(), "w");
 if(fh==NULL) error("Open file for output failed!\n");


/*
// Fit the logistic null modle
// Matrix Xnull;
// Vector ynull;
// GetNullData(pheno->affections[0], pheno->covariates, ynull, Xnull);

 LogisticRegressionScoreTest lrs;
 lrs.FitNullModel(Xnull, ynull, NROUNDS);
*/

// Process for groups one by one and output resutls to a file.
 for(int i=0; i<mg->count; i++)
 {
  //printf("Group %d\n", i);
  //printf("Chr=%d\n", mg->groups[mg->group_names[i]].pos[0]);
  
  std::string group_name = mg->group_names[i];
  int group_size = mg->groups[group_name].count;
  std::string var_chr;
  int var_pos;
  int counter = 0;
  double freq;
  MarkerGroupData mgd;
  mgd.geno.Dimension(pheno->pids.size(), group_size);
  mgd.geno.Set(-1.0);

  while(vin.readRecord())
   {
    VCFRecord &r = vin.getVCFRecord();
    VCFPeople &people = r.getPeople();

    if(people.size()==0) error("No individuals are present in the VCF file!\n");

    if(!toEstimateFreq && min_freq<1.0 && mg->groups[group_name].freq.size()==0)
    {
     if(std::string(r.getInfoTag("AF")).size()==0) {error("NO AF info available!\n"); }
     freq = atof(r.getInfoTag("AF"));
     if(freq>0.5) freq = 1-freq;
     if(freq>min_freq) continue;
    }

    var_chr = r.getChrom();
    var_pos = r.getPos();
    if(var_chr!=mg->groups[group_name].chr[counter] || var_pos!=mg->groups[group_name].pos[counter]) continue;

    StringArray gt;
    int gt_code;
    VCFIndividual* indv;


    for(int p=0; p<people.size(); p++)
      {   
            indv = people[p];
            if(pheno->pids_map.find( (*indv).getName()) == pheno->pids_map.end() ) 
	    {
	      continue;
   	      warning("Individual %s is not in pedfile!\n", (*indv).getName().c_str());
	    }

         gt.ReplaceTokens(String ((*indv)[0].toStr() ), ",");

	 if(gt[0]=="0/0") gt_code = 0;
	 else if(gt[0]=="0/1") gt_code = 1;
	 else if(gt[0]=="1/1") gt_code = 2; 
	 else error("No missing genotypes or non-biallelic markers are allowed for this version: %s %d", var_chr.c_str(), var_pos);

        if(mg->groups[group_name].freq[counter]>0.5) gt_code = 2 - gt_code;
	mgd.geno[pheno->pids_map[(*indv).getName()]][counter] = gt_code;

     } // End of for loop for all individuals

    counter++;
    if(group_size == counter) break;

   } // End of while loop to read in all records in a group
  
   if(toEstimateFreq)
   { mgd.EstimateAlleleFreq();
      if(min_freq<1.0) mgd.FilterGenotypeByFreq(min_freq);
   }
// Next we'll perform collapsing and tests  
  Matrix new_geno;
  mgd.WeightedSum(mg->groups[mg->group_names[i]].weight);
  mgd.Add1Col2Left(mgd.geno_ws, new_geno);
  //mgd.PrintMarkerData(mgd.geno);

// Write out grup name and sumary in the first column
 printf("%s",group_name.c_str());
for(int j=0; j<pheno->affectionCount; j++)
{
 mgd.CountRareVariants(pheno->affections[j]);
 printf("\t%d:%d\t%d:%d", mgd.case_carrier_count, mgd.ctrl_carrier_count, int(mgd.case_var_count), int(mgd.ctrl_var_count));
}
 
// Wal test
  if(par->wald)
  {
   LogisticRegression lr;
   for(int j=0; j<pheno->affectionCount; j++)
   {
    std::vector<int> comm_indices = ExtractCommonIndices(isGenotyped, pheno->affections[j], true);
    if(comm_indices.size()==0) { error("No individuals with both genotypes and affection status found for WALD test. Skip...\n"); break; }
    Matrix data;
    ExtractSubsetRows(data, new_geno, comm_indices);
    Vector status;
    ExtractSubsetRows(status, pheno->affections[j], comm_indices);

    Vector p, b;
    lr.FitLogisticModel(data, status, NROUNDS);  
    p=lr.GetAsyPvalue();
    b=lr.GetCovEst();
    OutputWaldRes2File(fh, b, p);
   }

   LinearRegression linr;
   for(int j=0; j<pheno->traitCount; j++)
   {
    std::vector<int> comm_indices = ExtractCommonIndices(isGenotyped, pheno->traits[j], false);
    if(comm_indices.size()==0) { error("No individuals with both genotypes and contiuous triats found for WALD. Skip...\n"); break; }
    Matrix data;
    ExtractSubsetRows(data, new_geno, comm_indices);
    Vector trait;
    ExtractSubsetRows(trait, pheno->traits[j], comm_indices);

    Vector p, b;
    linr.FitLinearModel(data, trait);  
    p=linr.GetAsyPvalue();
    //b=linr.GetCovEst();
    OutputWaldRes2File(fh, p, p);
   }
  }

// Score test
  if(par->score)
  {
   LogisticRegressionScoreTest lrs;
   std::vector<double> pvalues;
   for(int j=0; j<pheno->affectionCount; j++)
   {
    std::vector<int> comm_indices = ExtractCommonIndices(isGenotyped, pheno->affections[j], true);
    if(comm_indices.size()==0) { error("No indiviuals with both geotypes and affection status found for SCORE test. Skip...\n"); break; }
    Matrix data;
    ExtractSubsetRows(data, new_geno, comm_indices);
    Vector status;
    ExtractSubsetRows(status, pheno->affections[j], comm_indices);
    lrs.FitLogisticModel(data, status, 1, NROUNDS);
    //lrs.TestCovariate(Xnull, status, mgd.geno_ws);
    pvalues.push_back(lrs.pvalue);
   }
    OutputScoreRes2File(fh, pvalues);
  }

  Skat skat;
  Matrix geno_skat;
  mgd.Add1Col2Left(mgd.geno, geno_skat);
  //double pvalue = skat.CalculatePValue(pheno->affections[0], &y0, geno_skat, mg.weight);
  

// Write a new line and go to next group
  printf("\n");

 } // End of the analysis of a group

// Clsoe output file handler and the anlayses are done
fclose(fh);
}

void PedDataAnalyzer::OutputWaldRes2File(FILE *fh, Vector &beta, Vector &pvalues)
{
 for(int i=1; i<pvalues.Length(); i++)
 {
/*
  if(beta[i]<0.01 || beta[i]>100)
   printf("\t%.1e", beta[i]);
  else
   printf("\t%.2f", beta[i]);
*/
  if(pvalues[i]<0.001)
   printf("\t%.1e", pvalues[i]);
  else
   printf("\t%.3f", pvalues[i]);
 }
}

void PedDataAnalyzer::OutputScoreRes2File(FILE *fh, std::vector<double> &pvalues)
{
 for(int i=0; i<pvalues.size(); i++)
  if(pvalues[i]<0.001)
   printf("\t%.3f", pvalues[i]);
  else
    printf("\t%.3f", pvalues[i]);
}

void PedDataAnalyzer::ExtractSubsetRows(Matrix &new_m, Matrix &m, std::vector<int> &index)
{
 new_m.Dimension(index.size(), m.cols);
 for(int i=0; i<index.size(); i++)
  for(int j=0; j<m.cols; j++)
   new_m[i][j] = m[index[i]][j];
}

void PedDataAnalyzer::ExtractSubsetRows(Vector &new_v, Vector &v, std::vector<int> &index)
{
 new_v.Dimension(index.size());
 for(int i=0; i<index.size(); i++)
   new_v[i] = v[index[i]];
}

std::vector<int>  PedDataAnalyzer::ExtractCommonIndices(std::vector<bool> &isGenotyped, Vector &pheno, bool flag)
{
 std::vector<int> comm_indices;
 assert(isGenotyped.size()==pheno.Length());

 for(int i=0; i<isGenotyped.size(); i++)
{
  if(flag && isGenotyped[i] && ( pheno[i]==0 || pheno[i]==1 ))
   comm_indices.push_back(i);
  
  if(!flag && isGenotyped[i] && pheno[i]!=_NAN_)
   comm_indices.push_back(i);
}
 return(comm_indices);
}

std::vector<int>  PedDataAnalyzer::GetNullData(Vector &y, Matrix &Xcovariate, Vector &ynull, Matrix &Xnull)
{
 std::vector<int> comm_indices;

 for(int i=0; i<Xcovariate.rows; i++)
 {
  bool flag = true;
  if(y[i]!=0.0 && y[i]!=1.0) continue;
   for(int j=0; j<Xcovariate.cols; j++)
    if(Xcovariate[i][j]==_NAN_){ flag=false; break; }
   
  if(flag) comm_indices.push_back(i);
 }

 Xnull.Dimension(comm_indices.size(), Xcovariate.cols+1);
 for(int i=0; i<Xnull.rows; i++)
 {
   for(int j=0; j<Xnull.cols; j++)
    Xnull[i][j] = j==0 ? 1.0 : Xcovariate[comm_indices[i]][j-1];
   ynull[i] = y[comm_indices[i]];
 }
 return(comm_indices);
}

