#ifndef __PEDDATAANALYZER_H__
#define __PEDDATAANALYZER_H__

#include "Pedigree.h"
#include "PedPhenoData.h"
#include "MarkerGroup.h"
#include "MarkerGroupData.h"
#include "LinearRegression.h"
#include "LogisticRegression.h"
#include "CmdPar.h"
#include "Skat.h"

#include <cassert>
#include <string>
#include <set>
#include <map>
#include <vector>
#include <algorithm>
#include <iostream>
#include "Argument.h"
#include "IO.h"
#include "tabix.h"
#include "PeopleSet.h"
#include "RangeList.h"
#include "Utils.h"
#include "VCFUtil.h"

class PedDataAnalyzer
{
 public:
  PedPhenoData *pheno;
  MarkerGroup *mg;
  CmdPar *par;
  double min_freq;
  bool toEstimateFreq;
 public:
  PedDataAnalyzer();
 ~PedDataAnalyzer();
  void SetPedPhenoData(PedPhenoData *phenotypes) { pheno = phenotypes; }
  void SetMarkerGroup(MarkerGroup *m) { mg = m; }
  void SetMinFreq(double f){min_freq = f; }
  void SetCmdPar(CmdPar *cmdpar) { par = cmdpar; }
  void ExtractSubsetRows(Matrix&, Matrix &m, std::vector<int> &index);
  void ExtractSubsetRows(Vector &, Vector &v, std::vector<int> &index);
  std::vector<int> GetNullData(Vector& y, Matrix&, Vector &ynull, Matrix &Xnull);
  std::vector<int> ExtractCommonIndices(std::vector<bool> &isGenotyped, Vector &isDiagnosed, bool);
  void OpenVCF4Read(std::string);
  void LoadAndAnalyze(std::string, std::string);
  void OutputScoreRes2File(FILE *, std::vector<double>&);
  void OutputWaldRes2File(FILE *, Vector &, Vector&);
};

#endif
