#include "MarkerGroupData.h"
#include <cmath>

MarkerGroupData::MarkerGroupData()
{
 case_var_count=0.0;
 ctrl_var_count=0.0;
 case_carrier_count=0;
 ctrl_carrier_count=0;
 var_count=0;
 cum_freq = 0.0;
}

MarkerGroupData::~MarkerGroupData(){}

void MarkerGroupData::EstimateAlleleFreq(std::vector<bool>& isFounder)
{
 float total = 0.0;
 int genocount = 0;
 mono_count = poly_count = 0;
 cum_freq = 0.0;

 freq.resize(geno.cols, 0.0);
 count.resize(geno.cols, 0);

 for(int i=0; i<geno.cols; i++)
 {
  total = 0.0; 
  genocount = 0;

  for(int p=0; p<geno.rows; p++)
   if(isFounder[p] && int(geno[p][i])>=0)
   {
     total += geno[p][i];
     genocount++;
   }

  freq[i] = (total)/double(genocount*2);
  count[i] = int(total);
  if(freq[i]>0.5)
  { 
	freq[i] = 1-freq[i];
        count[i] = genocount*2-count[i];
	for(int p=0; p<geno.rows; p++)
	 if(int(geno[p][i])>=0) {
		geno[p][i] = 2-geno[p][i];
		phased_a1[p][i]=1-phased_a1[p][i];
		phased_a2[p][i]=1-phased_a2[p][i];
	 }
  }

  cum_freq += freq[i]>0 ? freq[i] : 0.0;
  if(total==0) mono_count++;
  else poly_count++;
 }
}

void MarkerGroupData::EstimateAlleleFreq_chrX(std::vector<bool>& isFounder, std::vector<int>& sexes)
{
 float total = 0.0;
 int genocount = 0;
 mono_count = poly_count = 0;
 cum_freq = 0.0;

 freq.resize(geno.cols, 0);
 count.resize(geno.cols, 0);

 for(int i=0; i<geno.cols; i++)
 {
  total = 0.0;
  genocount = 0;

  for(int p=0; p<geno.rows; p++)
   if(isFounder[p] && int(geno[p][i])>=0)
   {
     total += sexes[p]==1 ? geno[p][i]/2.0 : geno[p][i];
     genocount += sexes[p]==1 ? 0.5 : 1.0;
   }

  freq[i] = (total)/double(genocount*2);
  count[i] = int(total);
  if(freq[i]>0.5)
  {
	freq[i] = 1-freq[i];
        count[i] = genocount*2-count[i];
	for(int p=0; p<geno.rows; p++)
	 if(int(geno[p][i])>=0) geno[p][i] = 2-geno[p][i];
  }

  cum_freq += freq[i]>0 ? freq[i] : 0.0;
  if(total==0) mono_count++;
  else poly_count++;
 }
}

void MarkerGroupData::AssignWeights(double pseudo)
{
 if(freq.size()==0) error("Frequenncy vector is empty. Need to estimate freq first!\n");
 weight.resize(freq.size(), 0);
 for(int i=0; i<weight.size(); i++)
  if(freq[i]==0) weight[i] = 0.0;
  else weight[i] = 1/sqrt((freq[i]+pseudo)*(1-freq[i]-pseudo));
}

void MarkerGroupData::EstimateAlleleFreq(Vector &status)
{
 float total = 0.0;
 int genocount = 0;
 float total_case = 0.0;
 float total_ctrl = 0.0;
 int genocount_case = 0;
 int genocount_ctrl = 0;
 mono_count = poly_count = 0;

 freq.resize(geno.cols);
 count.resize(geno.cols);

 freq_case.resize(geno.cols);
 freq_ctrl.resize(geno.cols);
 count_case.resize(geno.cols);
 count_ctrl.resize(geno.cols);

 for(int i=0; i<geno.cols; i++)
 {
  total = 0.0; 
  genocount = 0;

  for(int p=0; p<geno.rows; p++)
   if(geno[p][i]>=0.0) 
   {
     total += geno[p][i];
     genocount++;
     if(status==0) { total_case += geno[p][i]; genocount_case++; }
     if(status==1) { total_ctrl += geno[p][i]; genocount_ctrl++; }
   }

  freq[i] = (total)/(genocount*2);
  count[i] = total;
  freq_case[i] = total_case/(genocount_case*2);
  freq_ctrl[i] = total_ctrl/(genocount_ctrl*2);
  count_case[i] = total_case;
  count_ctrl[i] = total_ctrl;

  if(freq[i]>0.5)
  { 
	freq[i] = 1-freq[i];
        count[i] = genocount*2-count[i];

	freq_case[i] = 1-freq_case[i];
        freq_ctrl[i] = 1-freq_ctrl[i];
        count_case[i] = genocount_case*2-count_case[i];
        count_ctrl[i] = genocount_ctrl*2-count_ctrl[i];

	for(int p=0; p<geno.rows; p++)
	 geno[p][i] = 2-geno[p][i];
  }
  if(total==0) mono_count++;
  else poly_count++;

 }
}

void MarkerGroupData::EstimateAlleleFreq_BLUE() {}

void MarkerGroupData::EstimateAlleleFreq_case()
{}

void MarkerGroupData::EstimateAlleleFreq_ctrl()
{}

// this filter should be called before others, and this doesn't update poly_count
void MarkerGroupData::FilterVariantsByDepth(double min, double max)
{
 double avg_dp;
 for(int i=0; i<geno.cols; i++) {
  avg_dp = double(depth[i])/double(depth_counter[i]);
  if(avg_dp>=min && avg_dp<=max || avg_dp>=min && max<=0.0) continue;

  for(int j=0; j<geno.rows; j++)
   geno[j][i] = -1.0;
 }
}

void MarkerGroupData::FilterGenotypeByFreq(double &min_freq, double &max_freq)
{
 for(int i=0; i<geno.cols; i++)
 {
  if(freq[i]<=max_freq && freq[i]>=min_freq) continue;

  if(freq[i]>0) {
   poly_count--;
   mono_count++;
   cum_freq -= freq[i]>0.0 ? freq[i] : 0.0;
   freq[i] = 0.0;
   count[i] = 0;

   for(int j=0; j<geno.rows; j++)
   {
     geno[j][i] = -1.0;
     phased_a1[j][i] = -1.0;
     phased_a2[j][i] = -1.0;
   }
  }
 }
 if(poly_count==0) cum_freq = 0.0;
}

void MarkerGroupData::FilterGenotypeByCount(int &min_count, int &max_count)
{
 for(int i=0; i<geno.cols; i++)
 {
  if(count[i]<=max_count && count[i]>=min_count) continue;
  freq[i] = 0.0;
  count[i] = 0;
  poly_count--;
  mono_count++;
  for(int j=0; j<geno.rows; j++)
   geno[j][i] = 0.0;
 }
}

void MarkerGroupData::WeightedSum(Matrix& wt)
{
 geno_ws.Product(geno, wt);
}

void MarkerGroupData::WeightedSum(std::vector<float>& w)
{
 Matrix wt;
 wt.Dimension(w.size(), 1);
 for(int i=0; i<w.size(); i++)
  wt[i][0] = w[i];
 geno_ws.Product(geno, wt);
}

void MarkerGroupData::SetCarriers()
{
 for(int i=0; i<geno_ws.rows; i++)
  for(int j=0; j<geno_ws.cols; j++)
   if(geno_ws[i][j] > 0.0) geno_ws[i][j] = 1.0;
}

void MarkerGroupData::CountRareVariants(Vector &status)
{
 case_var_count=0; ctrl_var_count=0; other_var_count=0;
 case_carrier_count=0; ctrl_carrier_count=0; other_carrier_count=0;

 std::vector<int> count;
 count.resize(geno.rows, 0);

 for(int i=0; i<geno.rows; i++)
  for(int j=0; j<geno.cols; j++)
   count[i] += geno[i][j]>0 ? int(geno[i][j]) : 0;

 for(int i=0; i<count.size(); i++)
 {
  if(int(status[i])==1) { case_var_count += count[i]; if(count[i]>0) case_carrier_count++; }
  else if(int(status[i])==0) { ctrl_var_count += count[i]; if(count[i]>0) ctrl_carrier_count++; }
  else { other_var_count += count[i]; if(count[i]>0) other_carrier_count++; }
 }

}

void MarkerGroupData::CountRareVariants_chrX(Vector &status, std::vector<int>& sexes)
{
 case_var_count=0; ctrl_var_count=0; other_var_count=0;
 case_carrier_count=0; ctrl_carrier_count=0; other_carrier_count=0;

 std::vector<int> count;
 count.resize(geno.rows, 0);

 for(int i=0; i<geno.rows; i++)
  for(int j=0; j<geno.cols; j++)
   count[i] += geno[i][j]>0 ? (sexes[i]==1 ? int(geno[i][j])/2 : int(geno[i][j])) : 0;

 for(int i=0; i<count.size(); i++)
 {
  if(int(status[i])==1) { case_var_count += count[i]; if(count[i]>0) case_carrier_count++; }
  else if(int(status[i])==0) { ctrl_var_count += count[i]; if(count[i]>0) ctrl_carrier_count++; }
  else { other_var_count += count[i]; if(count[i]>0) other_carrier_count++; }
 }
}

void MarkerGroupData::GetVarSummary(std::vector<bool>& isFounder)
{
 EstimateAlleleFreq(isFounder);

 cum_freq = 0.0;
 var_count = 0;
 for(int i=0; i<freq.size(); i++)
 {
  cum_freq += freq[i]>0.0 ? freq[i] : 0.0;
  if(freq[i]>0.0) var_count++;
 }
}

void MarkerGroupData::Add1Col2Left(Matrix &orig, Matrix &dest)
{
 dest.Dimension(orig.rows, orig.cols+1);
 for(int i=0; i<dest.rows; i++)
  for(int j=0; j<dest.cols; j++)
  {
   if(j==0) { dest[i][j] = 1.0; continue; }
   dest[i][j] = orig[i][j-1];
  }
}

void MarkerGroupData::PrintMarkerData()
{
 for(int i=0; i<geno.rows; i++)
 {
  for(int j=0; j<geno.cols; j++)
   printf("%f ", geno[i][j]);
  printf("\n");
 }
}

void MarkerGroupData::PrintMarkerData(Matrix &geno)
{
 for(int i=0; i<geno.rows; i++)
 {
  for(int j=0; j<geno.cols; j++)
   printf("%f ", geno[i][j]);
  printf("\n");
 }
}

