#ifndef __MARKERGROUPDATA_H__
#define __MARKERGROUPDATA_H__

#include "PedKinship.h"
#include "Pedigree.h"
#include "MathMatrix.h"
#include <vector>

class MarkerGroupData
{
 public:
  Matrix geno;
  Matrix geno_ws;
  Matrix phased_a1;
  Matrix phased_a2;
  PedKinship pedkin;
  bool isChrX;
  bool isChrY;
  bool isChrMT;
  std::vector<int> sexes;
  std::vector<int> depth;
  std::vector<int> depth_counter;

  std::vector<std::vector<std::pair<int,int> > > geno_phased;
  std::vector<float> weight;

  std::vector<float> freq;
  std::vector<int> count;
  std::vector<float> freq_case;
  std::vector<float> freq_ctrl;
  std::vector<int> count_case;
  std::vector<int> count_ctrl;

  float case_var_count; //these can be used for dosage as well
  float ctrl_var_count;
  float other_var_count;
  int case_carrier_count;
  int ctrl_carrier_count;
  int other_carrier_count;
  int var_count;
  float cum_freq;
  int cum_count;
  int mono_count;
  int poly_count;

 public:
  MarkerGroupData();
 ~MarkerGroupData();
  void EstimateAlleleFreq();
  void EstimateAlleleFreq(std::vector<bool>& isFounder);
  void EstimateAlleleFreq_chrX(std::vector<bool>& isFounder, std::vector<int>& sexes);
  void EstimateAlleleFreq(Vector& status);
  void EstimateAlleleFreq_case();
  void EstimateAlleleFreq_ctrl();
  void EstimateAlleleFreq_BLUE();
  void AssignWeights(double pseudo);
  void GetVarSummary(std::vector<bool>& isFounder);
  void FilterVariantsByDepth(double min, double max);
  void FilterGenotypeByFreq(double &min_freq, double &max_freq);
  void FilterGenotypeByCount(int &min_count, int &max_count);
  void WeightedSum(Matrix&);
  void WeightedSum(std::vector<float>&);
  void SetCarriers();
  void CountRareVariants(Vector&);
  void CountRareVariants_chrX(Vector &status, std::vector<int>& sexes);
  void Add1Col2Left(Matrix &, Matrix &);
  void PrintMarkerData();
  void PrintMarkerData(Matrix&);
  void Clear();
};

#endif
