#include <iostream>
#include <cstdlib>
#include <iostream>
#include <iomanip>
#include <cmath>
#include <ctime>
#include "PedDataAnalyzer.h"
#include "rvgs.h"

using namespace std;

#include "cdflib.hpp"

PedDataAnalyzer::PedDataAnalyzer()
{
 pheno = NULL;
 mg = NULL;
 max_freq = -1.0;
 min_freq = -1.0;
 toEstimateFreq = false;
 GT_idx = -1;
 DP_idx = -1;
 GQ_idx = -1;
 affection_index = 0;
}

PedDataAnalyzer::~PedDataAnalyzer(){
}

void PedDataAnalyzer::MapPID2Traverse()
{
  pid2traverse.clear();
  pids.clear();
  traverse2pid.clear();

  for(int i=0; i<ped->familyCount; i++)
    for(int j=0; j<ped->families[i]->count; j++)
      {
        std::pair<int, int> traverse_pair;
        traverse_pair.first = i;
        traverse_pair.second = j;

        Person *p;
        p = ped->persons[ped->families[i]->path[j]];
        pid2traverse[std::string(p->pid.c_str())] = traverse_pair;
        traverse2pid[traverse_pair] = std::string(p->pid.c_str());
        pids.push_back(std::string(p->pid.c_str()));
      }
}

void PedDataAnalyzer::LoadAndAnalyze(std::string file, std::string outfile)
{
 VCFInputFile vin(file.c_str());
 if(pheno->pids.size()==0) pheno->GetPIDs();
 vin.includePeople(pheno->pids);

// Set flags to indicate the availability of genotype data for all individauls in the pedfile
 isGenotyped.clear();
 isGenotyped.resize(pheno->nIndividuals);

 std::vector<std::string> people;
 VCFHeader* header = vin.getVCFHeader();
 header->getPeopleName(&people);

 int isGenotypedCount = 0;
 for(int p=0; p<people.size(); p++)
 {
  if(pheno->pids_map.find(people[p]) == pheno->pids_map.end() ) continue;
  isGenotyped[pheno->pids_map[people[p]]] = true;
  isGenotypedCount++;
 }

 fprintf(stderr, "Number of samples being analyzed: %d\n\n", isGenotypedCount);

 //Construct NULL matrix with both phenotype, genotype and covariate data
 pheno->SetIsGenotyped(isGenotyped);
 pheno->GetNullData();

 std::string outch = outfile+".ch";
 CH = fopen(outch.c_str(), "w");

 outstream.open (outfile.c_str());
 if (outstream.fail()) error("Open output file %f failed!\n", outfile.c_str());
 outstream << "Chr\tGroup\tnVar\tmaf_sum\tcarrier-ca:co:unphe\tallele-ca:co:unphe\tfa-t:nt\tmo-t:nt";
 if(par->all) {
  outstream << "\tp_s\tz_s\tp_ws\tz_ws\tp_c\tz_c\tp_ch\tz_ch\tp_ch_w\tz_ch_w\tp_2a_add\tz_2a_add";
  if(par->permute>0) outstream<< "\tep_s\tep_ws\tep_c\tep_ch\tep_ch_w";
  if(par->ignore_mi_trio) outstream <<"\t"<<"trios_removed";
  outstream << endl;
 }
 else {
  if(par->sum) outstream << "\t" << "p_s" << "\t" << "z_s" << par->permute>0 ? "ep_s" : "";
  if(par->wss) outstream << "\t" << "p_ws" << "\t" << "z_ws" << par->permute>0 ? "ep_ws" : "";
  if(par->collapsing) outstream << "\t" << "p_c" << "\t" << "z_c" << par->permute>0 ? "ep_c" : "";
  if(par->comphet) outstream << "\t" << "p_ch" << "\t" << "z_ch" << par->permute>0 ? "ep_ch" : "";
  if(par->comphet_weight) outstream << "\t" << "p_ch_w" << "\t" << "z_ch_w" << par->permute>0 ? "ep_ch_w" : "";
  if(par->ignore_mi_trio) outstream <<"\t"<<"trios_removed";
  outstream << endl;
 }

 // set disease index to analyze
 if(par->disease.Length()>0){
  bool affectionFound = false;
  for(int i=0; i<pheno->ped->affectionNames.Length(); i++)
   if(pheno->ped->affectionNames[i]==par->disease) { affection_index = i; affectionFound = true; break; }
  if(affectionFound==false) warning("Disease %s is not found in the dat file. The first affection status is used\n");
 }

 std::vector<std::vector<double> > permute_stats;
 if(par->permute>0) permute_stats.resize(par->permute);

 // Process for groups one by one and output results to a file.
 for(int i=0; i<mg->count; i++)
 {
  current_group_index = i;
  group_name = mg->group_names[i];
  std::string chr_str = mg->groups[group_name].chr_str; // all chr in the group
  group_size = mg->groups[group_name].count;
  std::string var_chr;
  int var_pos;
  int counter = 0;
  double freq;
  mgd.geno.Dimension(pheno->pids.size(), group_size);
  mgd.geno.Set(-1.0);
  mgd.phased_a1.Dimension(pheno->pids.size(), group_size);
  mgd.phased_a2.Dimension(pheno->pids.size(), group_size);
  mgd.phased_a1.Set(-1.0);
  mgd.phased_a2.Set(-1.0);
  mgd.depth.resize(group_size, 0);
  mgd.depth_counter.resize(group_size, 0);

 for(int v=0; v<group_size; v++)
 {
  //the following line is to allow variants from multiple chromsomes to be in the same group
  vin.setRange(mg->groups[group_name].chr[v].c_str(), (unsigned int)mg->groups[group_name].pos[v], (unsigned int)mg->groups[group_name].pos[v]);

  bool recordFound = false;
  bool missing = false;
  std::string pass, alt;
  int qual;

  while(vin.readRecord())
   {
    VCFRecord &r = vin.getVCFRecord();
    VCFPeople &people = r.getPeople();

    if(GT_idx<0)
      {
        GT_idx = r.getFormatIndex("GT");
        if(GT_idx<0) error("NO GT field was found in FORMAT. Please check the vcf file at chr:%s and position:%d", r.getChrom(), r.getPos());
      }

    if(DP_idx<0) DP_idx = r.getFormatIndex("DP");
    if(GQ_idx<0) GQ_idx = r.getFormatIndex("GQ");

    if(people.size()==0) error("No individuals are present in the VCF file!\n");

    var_chr = r.getChrom();
    var_pos = r.getPos();
    if(var_chr!=mg->groups[group_name].chr[v] || var_pos!=mg->groups[group_name].pos[v]) continue;

    qual = atoi(r.getQual());
    if(par->min_qual>0 && qual>0 && qual<par->min_qual) continue;

    pass = std::string(r.getFilt());
    if(par->pass && pass.compare("PASS")!=0) continue;

    alt = std::string(r.getAlt());
    if(alt.find(',')!=std::string::npos) continue;

    StringArray gt;
    int gt_code;
    VCFIndividual* indv;
    int dp, gq;
    int a1_code, a2_code;

    for(int p=0; p<people.size(); p++)
     {
       indv = people[p];
       if(pheno->pids_map.find( (*indv).getName()) == pheno->pids_map.end() )  continue;

       std::string gt = (*indv).get(GT_idx, &missing).toStr();

       if(gt.compare("./.")==0 || gt.compare(".")==0 || gt.compare(".|.")==0 ) { gt_code = -1; a1_code=a2_code=-1; if(par->missing_as_ref) { gt_code = 0; a1_code=a2_code = 0; }}
       else if(gt.size()==3) {
	{
	  if(gt.compare("0/0")==0 || gt.compare("0|0")==0) gt_code = 0;
          else if(gt.compare("0/1")==0 || gt.compare("0|1")==0 || gt.compare("1|0")==0) gt_code = 1;
          else if(gt.compare("1/1")==0 || gt.compare("1|1")==0) gt_code = 2;
         }
	if(par->phased) {
	  if(gt.compare("0|0")==0) a1_code = a2_code = 0;
          else if(gt.compare("0|1")==0) { a1_code = 0; a2_code = 1;}
          else if(gt.compare("1|0")==0) { a1_code = 1; a2_code = 0; }
          else if(gt.compare("1|1")==0) a1_code = a2_code = 1;
        }
       }
       else if(gt.size()==1) {
	 if(gt.compare("0")==0) gt_code = 0;
	 else if(gt.compare("1")==0) gt_code = 2;

	if(par->phased) a1_code = a2_code = -1;
       }
       else  fprintf(stderr, "GT format is not supported and ignored: %s\n", gt.c_str());

       mgd.geno[pheno->pids_map[(*indv).getName()]][v] = double(gt_code);
       mgd.phased_a1[pheno->pids_map[(*indv).getName()]][v] = double(a1_code);
       mgd.phased_a2[pheno->pids_map[(*indv).getName()]][v] = double(a2_code);

      if(DP_idx>0) { dp=(*indv).get(DP_idx, &missing).toInt(); if(!missing) { mgd.depth[v]+=dp; mgd.depth_counter[v]++; } if(!missing && dp<par->min_dp) mgd.geno[pheno->pids_map[(*indv).getName()]][v]  = -1.0; }
      if(GQ_idx>0) { gq=(*indv).get(GQ_idx, &missing).toInt(); if(!missing && gq<par->min_gq) mgd.geno[pheno->pids_map[(*indv).getName()]][v]  = -1.0; }

     } // end of for loop for all individuals
    recordFound = true;
   } // end of while loop to read in all records in a range

   if(recordFound==false)
   {
     if(mg->groups[group_name].weight.size()>0)  mg->groups[group_name].weight[v] = 0.0;
     mg->groups[group_name].weight_one[v] = 0.0;
     if(mg->groups[group_name].freq.size()>0)  mg->groups[group_name].freq[v] = 0.0;
     fprintf(stderr, "WARNING: record was not found in VCF or did not meet filtering criteria, skipping... %s:%d\n", mg->groups[group_name].chr[v].c_str(), mg->groups[group_name].pos[v]);
   }

 } // end of reading all records in a group


    // filter variant by avg depth
    if(DP_idx>0 && (par->min_avg_depth>0.0 || par->max_avg_depth>0.0)) mgd.FilterVariantsByDepth(par->min_avg_depth, par->max_avg_depth);

    mgd.isChrX = mg->groups[group_name].isChrX;
    mgd.isChrY = mg->groups[group_name].isChrY;
    mgd.isChrMT = mg->groups[group_name].isChrMT;

    //estimate freq even if privided in the group file. this estimated will be used for minor allele coding
    if(!mgd.isChrX && !mgd.isChrY && !mgd.isChrMT) mgd.EstimateAlleleFreq(pheno->isFounder);
    if(mgd.isChrX) mgd.EstimateAlleleFreq_chrX(pheno->isFounder, pheno->sexes); //chrX

    // assign weight according to 1/sqrt(freq*(1-freq)) with pseudo count to avoid zero
    if(mg->groups[group_name].weight.size()==0) {
    	 mgd.AssignWeights(pheno->pseudo_freq);
	 mg->groups[group_name].weight = mgd.weight;
     }

    //filter variants with estimated freq cutoffs if freq is not provied in the group file
   if(mg->has_freq==false)
    if(par->max_freq<1.0 || par->min_freq>0.0) mgd.FilterGenotypeByFreq(par->min_freq, par->max_freq);
   // the follow is the filtering based on parental allele account. not easy to use, so disable this for now
   //if(par->max_count>0 && par->min_count>=0 && par->min_count<=par->max_count) mgd.FilterGenotypeByCount(par->min_count, par->max_count);

   // write out group name and variant counting summary
   outstream << chr_str << "\t" << group_name << "\t" << mgd.poly_count << "\t" << mgd.cum_freq << "\t";
   if(!mgd.isChrX && !mgd.isChrY && !mgd.isChrMT) mgd.CountRareVariants(pheno->affections[affection_index]);
   if(mgd.isChrX) mgd.CountRareVariants_chrX(pheno->affections[affection_index], pheno->sexes);
   outstream << mgd.case_carrier_count << ":" << mgd.ctrl_carrier_count << ":" << mgd.other_carrier_count <<"\t" <<int(mgd.case_var_count) << ":" << int(mgd.ctrl_var_count) << ":" << int(mgd.other_var_count);

   // performe various forms of TDT and output results to a file
   Collapsing_TDT();

 } // end of the analysis of a group

  // close output file handler and the anlayses are done
  outstream.close();
  fclose(CH);
}

void PedDataAnalyzer::Collapsing_TDT()
{
 int f1,f2,m1,m2,c1,c2;
 std::pair<int, int> zero_pair;
 mgd.geno_phased.resize(pheno->pids.size());
 for(int i=0; i<mgd.geno_phased.size(); i++)
  mgd.geno_phased[i].resize(group_size, zero_pair);

 std::vector<double> stats;
 stats.resize(16);

 ignore_mi_trio_cnt = 0;

 for(int i=0; i<mgd.geno_phased.size(); i++)
 {
  int sex = pheno->sexes[i];
  int fa_index = pheno->pids_fa_index[pheno->pids[i]];
  int mo_index = pheno->pids_mo_index[pheno->pids[i]];

  if(fa_index<0 || mo_index<0) continue; //ignore founders
  if(int(pheno->affections[affection_index][i]) !=1) continue; //child not affected. 0=normal 1=affected

  if(par->phased && !mgd.isChrX && !mgd.isChrY && !mgd.isChrMT) //phased, and fill in parental transmitted haplotypes assuming the order of transmission is unknown
  {
   std::vector<bool> fa_trans_1, fa_trans_2, mo_trans_1, mo_trans_2;
   fa_trans_1.resize(mgd.geno_phased[i].size(), false);
   fa_trans_2.resize(mgd.geno_phased[i].size(), false);
   mo_trans_1.resize(mgd.geno_phased[i].size(), false);
   mo_trans_2.resize(mgd.geno_phased[i].size(), false);


   int data_cnt = 0;
   for(int j=0; j<mgd.geno_phased[i].size(); j++)
   {
    if(mgd.phased_a1[i][j]<0 || mgd.phased_a2[i][j]<0 || mgd.phased_a1[fa_index][j]<0 || mgd.phased_a2[fa_index][j]<0 || mgd.phased_a1[mo_index][j]<0 || mgd.phased_a2[mo_index][j]<0)
    {
     mgd.phased_a1[i][j]  = mgd.phased_a2[i][j] = -1.0;
     mgd.phased_a1[fa_index][j] = mgd.phased_a2[fa_index][j] = -1.0;
     mgd.phased_a1[mo_index][j] = mgd.phased_a2[mo_index][j] = -1.0;
     continue;
    }
    if(mgd.phased_a1[i][j] == mgd.phased_a1[fa_index][j]) fa_trans_1[j] = true;
    if(mgd.phased_a1[i][j] == mgd.phased_a2[fa_index][j]) fa_trans_2[j] = true;
    if(mgd.phased_a2[i][j] == mgd.phased_a1[mo_index][j]) mo_trans_1[j] = true;
    if(mgd.phased_a2[i][j] == mgd.phased_a2[mo_index][j]) mo_trans_2[j] = true;

//printf("%d %d %d %d\n", fa_trans_1[0], fa_trans_2[0], mo_trans_1[0], mo_trans_2[0]);

    data_cnt++;
   }

   bool fa1 = true;
   bool fa2 = true;
   bool mo1 = true;
   bool mo2 = true;
   for(int j=0; j<mgd.geno_phased[i].size(); j++)
   {
    if(mgd.phased_a1[i][j]<0.0) continue;
    if(fa_trans_1[j]==false) fa1 = false;
    if(fa_trans_2[j]==false) fa2 = false;
    if(mo_trans_1[j]==false) mo1 = false;
    if(mo_trans_2[j]==false) mo2 = false;
   }

   if((fa1==false && fa2==false || mo1==false && mo2==false) && data_cnt>0) error("Phased genotypes are not compatable. Are there de novo mutations?\n");

   for(int j=0; j<mgd.geno_phased[i].size(); j++)
   {
    mgd.geno_phased[fa_index][j].first = fa1==true ? mgd.phased_a1[fa_index][j] : mgd.phased_a2[fa_index][j];
    mgd.geno_phased[fa_index][j].second = fa1==true ? mgd.phased_a2[fa_index][j] : mgd.phased_a1[fa_index][j];
    mgd.geno_phased[mo_index][j].first = mo1==true ? mgd.phased_a1[mo_index][j] : mgd.phased_a2[mo_index][j];
    mgd.geno_phased[mo_index][j].second = mo1==true ?  mgd.phased_a2[mo_index][j] : mgd.phased_a1[mo_index][j];

    if(mgd.geno_phased[fa_index][j].first<0.0) mgd.geno_phased[fa_index][j].first=0.0;
    if(mgd.geno_phased[fa_index][j].second<0.0) mgd.geno_phased[fa_index][j].second=0.0;
    if(mgd.geno_phased[mo_index][j].first<0.0) mgd.geno_phased[mo_index][j].first=0.0;
    if(mgd.geno_phased[mo_index][j].second<0.0) mgd.geno_phased[mo_index][j].second=0.0;
   }

  } //end of phased for a trio

  else //unphased, and phased by transmission
  for(int j=0; j<mgd.geno_phased[i].size(); j++)
  {
   int c_gt = int(mgd.geno[i][j]);
   int f_gt = int(mgd.geno[fa_index][j]);
   int m_gt = int(mgd.geno[mo_index][j]);

   if(!mgd.isChrX && !mgd.isChrY && !mgd.isChrMT)
   {
    f1=f2=m1=m2=c1=c2=0;
    // -1: all hetero, -2: de nov mutations, -3: same homo in parents, -4 diff homo in parents, -5: missing data in one of the three. All these should be removed for TDT
    if(f_gt==0 && m_gt==0 && c_gt==0) { c1=-3; c2=-3; f1=f2=m1=m2=c1=f2=0; }
    if(f_gt==0 && m_gt==0 && c_gt==1) { c1=-2; c2=-2; }
    if(f_gt==0 && m_gt==0 && c_gt==2) { c1=-2; c2=-2; }
    if(f_gt==0 && m_gt==1 && c_gt==0) { c1=0; c2=0; f1=0; f2=0; m1=0; m2=1; }
    if(f_gt==0 && m_gt==1 && c_gt==1) { c1=0; c2=1; f1=0; f2=0; m1=1; m2=0; }
    if(f_gt==0 && m_gt==1 && c_gt==2) { c1=c2=-2; }
    if(f_gt==0 && m_gt==2 && c_gt==0) { c1=-2; c2=-2; }
    if(f_gt==0 && m_gt==2 && c_gt==1) { c1=-4; c2=-4; f1=0; f2=0; m1=1; m2=1; }
    if(f_gt==0 && m_gt==2 && c_gt==2) { c1=c2=-2; }
    if(f_gt==1 && m_gt==0 && c_gt==0) { c1=0; c2=0; f1=0; f2=1; m1=0; m2=0; }
    if(f_gt==1 && m_gt==0 && c_gt==1) { c1=1; c2=0; f1=1; f2=0; m1=0; m2=0; }
    if(f_gt==1 && m_gt==0 && c_gt==2) { c1=-2; c2=-2; }
    if(f_gt==1 && m_gt==1 && c_gt==0) { c1=0; c2=0; f1=0; f2=1; m1=0; m2=1; }
    if(f_gt==1 && m_gt==1 && c_gt==1) { c1=-1; c2=-1; f1=0; f2=1; m1=1; m2=0; if(Uniform(0.0,1.0)<0.5) {f1=1; f2=0; m1=0; m2=1; } } // unable to phase by transmission. will cause type I error inflation for common variants
    if(f_gt==1 && m_gt==1 && c_gt==2) { c1=1; c2=1; f1=1; f2=0; m1=1; m2=0; }
    if(f_gt==1 && m_gt==2 && c_gt==0) { c1=-2; c2=-2; }
    if(f_gt==1 && m_gt==2 && c_gt==1) { c1=0; c2=1; f1=0; f2=1; m1=1; m2=1; }
    if(f_gt==1 && m_gt==2 && c_gt==2) { c1=1; c2=1; f1=1; f2=0; m1=1; m2=1; }
    if(f_gt==2 && m_gt==0 && c_gt==0) { c1=-2; c2=-2; }
    if(f_gt==2 && m_gt==0 && c_gt==1) { c1=-4; c2=-4; f1=1; f2=1; m1=0; m2=0; }
    if(f_gt==2 && m_gt==0 && c_gt==2) { c1=-2; c2=-2; }
    if(f_gt==2 && m_gt==1 && c_gt==0) { c1=c2=-2; }
    if(f_gt==2 && m_gt==1 && c_gt==1) { c1=1; c2=0; f1=1; f2=1; m1=0; m2=1; }
    if(f_gt==2 && m_gt==1 && c_gt==2) { c1=1; c2=1; f1=1; f2=1; m1=1; m2=0; }
    if(f_gt==2 && m_gt==2 && c_gt==0) { c1=-2; c1=-2; }
    if(f_gt==2 && m_gt==2 && c_gt==1) { c1=-2; c2=-2; }
    if(f_gt==2 && m_gt==2 && c_gt==2) { c1=-3; c2=-3; f1=f2=m1=m2=c1=c2=1; }

    if(f_gt<0 || m_gt<0 || c_gt<0) { c1=-5; c2=-5; f1=f2=m1=m2=0; }

//if(c1<0 && c1!=-3 || c2<0 && c2!=-3) printf("%d %d : %d %d %d\n", c1,c2, f_gt, m_gt, c_gt);

    mgd.geno_phased[fa_index][j].first = f1;
    mgd.geno_phased[fa_index][j].second = f2;
    mgd.geno_phased[mo_index][j].first = m1;
    mgd.geno_phased[mo_index][j].second = m2;
    mgd.geno_phased[i][j].first = c1;
    mgd.geno_phased[i][j].second = c2;

//printf("%d-%d-%d:%d|%d:%d|%d:%d|%d ", f_gt, m_gt, c_gt, f1,f2,m1,m2, c1,c2);

   }

   if(mgd.isChrX)
   {
    f1=f2=m1=m2=c1=c2=0;
    // -2: de novo mutations, -3: same homo in parents, -4, diff homo in parents, -6: missing data in one of the three, -5 invalid. All these should be removed for TDT
    if(f_gt==0 && m_gt==0 && c_gt==0) { c1=c2=-3; f1=f2=m1=m2=0; }
    if(f_gt==0 && m_gt==0 && c_gt==1) { c1=c2=-2; }
    if(f_gt==0 && m_gt==0 && c_gt==2) { c1=c2=-2; }
    if(f_gt==0 && m_gt==1 && c_gt==0) { f1=f2=0; m1=0; m2=1; c1=c2=0; }
    if(f_gt==0 && m_gt==1 && c_gt==1) { if(sex==1) c1=c2=-5; if(sex==2) { f1=f2=0; m1=1; m2=0; c1=0; c2=1; } }
    if(f_gt==0 && m_gt==1 && c_gt==2) { if(sex==1) {f1=f2=0; m1=1; m2=0; c1=0; c2=1;} if(sex==2) c1=c2=-2; }
    if(f_gt==0 && m_gt==2 && c_gt==0) { c1=c2=-2; }
    if(f_gt==0 && m_gt==2 && c_gt==1) { if(sex==1) c1=c2=-5; if(sex==2) { f1=f2=0; m1=m2=1; c1=-4; c2=-4;} }
    if(f_gt==0 && m_gt==2 && c_gt==2) { if(sex==1) { f1=f2=0; m1=m2=1; c1=-4; c2=-4; }  if(sex==2) c1=c2=-2; }
    if(f_gt==1 && m_gt==0 && c_gt==0) { c1=c2=-5; }
    if(f_gt==1 && m_gt==0 && c_gt==1) { c1=c2=-5; }
    if(f_gt==1 && m_gt==0 && c_gt==2) { c1=c2=-5; }
    if(f_gt==1 && m_gt==1 && c_gt==0) { c1=c2=-5; }
    if(f_gt==1 && m_gt==1 && c_gt==1) { c1=c2=-5; }
    if(f_gt==1 && m_gt==1 && c_gt==2) { c1=c2=-5; }
    if(f_gt==1 && m_gt==2 && c_gt==0) { c1=c2=-5; }
    if(f_gt==1 && m_gt==2 && c_gt==1) { c1=c2=-5; }
    if(f_gt==1 && m_gt==2 && c_gt==2) { c1=c2=-5; }
    if(f_gt==2 && m_gt==0 && c_gt==0) { if(sex==1) {f1=f2=1; m1=m2=0; c1=c2=-4; } if(sex==2) c1=c2=-2; }
    if(f_gt==2 && m_gt==0 && c_gt==1) { if(sex==1) c1=c2=-5; if(sex==2) {f1=f2=1; m1=m2=0; c1=-4; c2=-4; } }
    if(f_gt==2 && m_gt==0 && c_gt==2) { c1=c2=-2; }
    if(f_gt==2 && m_gt==1 && c_gt==0) { if(sex==1) { f1=f2=1; m1=0; m2=1; c1=0; c2=0; } if(sex==2) c1=c2=-2; }
    if(f_gt==2 && m_gt==1 && c_gt==1) { if(sex==1) { c1=c2=-5; } if(sex==2) {f1=f2=1; m1=0; m2=1; c1=1; c2=0; } }
    if(f_gt==2 && m_gt==1 && c_gt==2) { f1=f2=1; m1=1; m2=0; if(sex==1) { c1=0; c2=1; } if(sex==2) c1=c2=1; }
    if(f_gt==2 && m_gt==2 && c_gt==0) { c1=c2=-2; }
    if(f_gt==2 && m_gt==2 && c_gt==1) { if(sex==1) c1=c2=-5; if(sex==2) c1=c2=-2; }
    if(f_gt==2 && m_gt==2 && c_gt==2) { c1=c2=-3; f1=f2=m1=m2=1; }

    if(f_gt<0 || m_gt<0 || c_gt<0) { c1=c2=-6; f1=f2=m1=m2=0; }

    mgd.geno_phased[fa_index][j].first = f1;
    mgd.geno_phased[fa_index][j].second = f2;
    mgd.geno_phased[mo_index][j].first = m1;
    mgd.geno_phased[mo_index][j].second = m2;
    mgd.geno_phased[i][j].first = c1;
    mgd.geno_phased[i][j].second = c2;

   }

  } //for all markers in a single trio
 } // for all trios;

//printf("\n");

 CalcTDTStats(stats, false);

 double f1_s_total = stats[10];
 double f2_s_total = stats[11];
 double m1_s_total = stats[12];
 double m2_s_total = stats[13];

 double p_s = stats[0];
 double z_s = stats[1];
 double p_ws = stats[2];
 double z_ws = stats[3];
 double p_c = stats[4];
 double z_c = stats[5];
 double p_ch = stats[6];
 double z_ch = stats[7];
 double p_ch_w = stats[8];
 double z_ch_w = stats[9];

 double p_2a_additive = stats[14];
 double z_2a_additive = stats[15];

 // these variables are for permutations
 double s,ws,c,ch,ch_w, e_p_s, e_p_ws, e_p_c, e_p_ch, e_p_ch_w;
 s=ws=c=ch=ch_w=0.0;

 if(par->permute>0)
 {
  for(int p=0; p<par->permute; p++)
  {
   CalcTDTStats(stats, true);

   if(abs(z_s) < abs(stats[1])) s+=1.0; //if(abs(z_s) == abs(stats[1])) s+=0.5;
   if(abs(z_ws) < abs(stats[3])) ws+=1.0; //if(abs(z_s) == abs(stats[3])) ws+=0.5;
   if(abs(z_c) < abs(stats[5])) c+=1.0; //if(abs(z_s) == abs(stats[5])) c+=0.5;
   if(abs(z_ch) < abs(stats[7])) ch+=1.0; //if(abs(z_s) == abs(stats[7])) ch+=0.5;
   if(abs(z_ch_w) < abs(stats[9])) ch_w+=1.0; //if(abs(z_s) == abs(stats[9])) ch_w+=0.5;
  }
  e_p_s = isnan(z_s) ? NAN : s/par->permute;
  e_p_ws = isnan(z_ws) ? NAN : ws/par->permute;
  e_p_c = isnan(z_c) ? NAN : c/par->permute;
  e_p_ch = isnan(z_ch) ? NAN : ch/par->permute;
  e_p_ch_w = isnan(z_ch_w) ? NAN : ch_w/par->permute;
 }

 outstream << "\t" << f1_s_total << ":" << f2_s_total;
 outstream << "\t" << m1_s_total << ":" << m2_s_total;

 if(par->all)
 {
    outstream << "\t" << p_s << "\t" <<  z_s << "\t" << p_ws << "\t" << z_ws << "\t" << p_c << "\t" << z_c << "\t" << p_ch << "\t" << z_ch << "\t" << p_ch_w << "\t" << z_ch_w << "\t" << p_2a_additive <<"\t" << z_2a_additive;
    if(par->permute) outstream <<"\t" << e_p_s << "\t" << e_p_ws << "\t" << e_p_c << "\t" << e_p_ch << "\t" << e_p_ch_w;

    if(par->ignore_mi_trio) outstream <<"\t"<<ignore_mi_trio_cnt;

    outstream << endl;
 }
 else {
  if(par->sum) { outstream << "\t" << p_s << "\t" << z_s; if(par->permute>0) outstream<<"\t"<<e_p_s; }
  if(par->wss) { outstream << "\t" << p_ws << "\t" << z_ws; if(par->permute>0) outstream<<"\t"<<e_p_ws; }
  if(par->collapsing) { outstream << "\t" << p_c << "\t" << z_c; if(par->permute>0) outstream<<"\t"<<e_p_c; }
  if(par->comphet) { outstream << "\t" << p_ch << "\t" << z_ch; if(par->permute>0) outstream<<"\t"<<e_p_ch; }
  if(par->comphet_weight) { outstream << "\t" << p_ch_w << "\t" << z_ch_w; if(par->permute>0) outstream<<"\t"<<e_p_ch_w; }

  if(par->ignore_mi_trio) outstream <<"\t"<<ignore_mi_trio_cnt;

  outstream << endl;
 }
}

void PedDataAnalyzer::CalcTDTStats(std::vector<double>& stats, bool permute)
{
  //after phasing for an affected offspring, calculate the statitiscs for this trio and move to another affected offspring.
  //phasing may be different for different offspring, so the statistic need to be calculated for each affected offspring
  double f1_ws = 0.0;
  double f2_ws = 0.0;
  double m1_ws = 0.0;
  double m2_ws = 0.0;

  double f1_s = 0.0;
  double f2_s = 0.0;
  double m1_s = 0.0;
  double m2_s = 0.0;

  double f1_ws_total = 0.0;
  double f2_ws_total = 0.0;
  double m1_ws_total = 0.0;
  double m2_ws_total = 0.0;

  double f1_s_total = 0.0;
  double f2_s_total = 0.0;
  double m1_s_total = 0.0;
  double m2_s_total = 0.0;

  double tdt_s_total = 0.0;
  double tdt_s_total_var = 0.0;
  double tdt_c_total = 0.0;
  double tdt_c_total_var = 0.0;
  double tdt_ws_total = 0.0;
  double tdt_ws_total_var = 0.0;
  double tdt_ch_total = 0.0;
  double tdt_ch_total_var = 0.0;
  double tdt_ch_w_total = 0.0;
  double tdt_ch_w_total_var = 0.0;
  double tdt_2a_additive_total = 0.0;
  double tdt_2a_additive_total_var = 0.0;

 int nVar = mgd.geno_phased[0].size();
 std::vector<int> f1,f2,m1,m2,c1,c2;

 ignore_mi_trio_cnt = 0;

 for(int i=0; i<mgd.geno_phased.size(); i++)
 {
  int sex = pheno->sexes[i];
  int fa_index = pheno->pids_fa_index[pheno->pids[i]];
  int mo_index = pheno->pids_mo_index[pheno->pids[i]];

  if(fa_index<0 || mo_index<0) continue; //ignore founders
  if(int(pheno->affections[affection_index][i]) !=1) continue; //child not affected. 0=normal 1=affected

  f1_ws = 0.0;
  f2_ws = 0.0;
  m1_ws = 0.0;
  m2_ws = 0.0;

  f1_s = 0.0;
  f2_s = 0.0;
  m1_s = 0.0;
  m2_s = 0.0;

  bool remove_trio = false;

  f1.resize(nVar, 0);
  f2.resize(nVar, 0);
  m1.resize(nVar, 0);
  m2.resize(nVar, 0);
  c1.resize(nVar, 0);
  c2.resize(nVar, 0);

  for(int j=0; j<mgd.geno_phased[i].size(); j++)
  {
  // ignore de novo mutation and non-informative sites
  //if(!par->ignore_noninformative && (mgd.geno_phased[i][j].first==-3 || mgd.geno_phased[i][j].first==-4)) {} //keep non-informative single markers that are only phaseable. they may be informative for cmc and CH. all het are removed due to lack of phasing info
  //else 
//  if(mgd.geno_phased[i][j].first<0 || mgd.geno_phased[i][j].second<0) continue;

  if(par->ignore_mi_trio && mgd.geno_phased[i][j].first==-2) { ignore_mi_trio_cnt++; remove_trio = true; break;}
  if(mgd.geno_phased[i][j].first==-2 || mgd.geno_phased[i][j].first==-5) continue;

  f1[j] = mgd.geno_phased[fa_index][j].first;
  f2[j] = mgd.geno_phased[fa_index][j].second;
  m1[j] = mgd.geno_phased[mo_index][j].first;
  m2[j] = mgd.geno_phased[mo_index][j].second;
  c1[j] = mgd.geno_phased[i][j].first;
  c2[j] = mgd.geno_phased[i][j].second;

  f1_ws += f1[j]*mg->groups[group_name].weight[j];
  f2_ws += f2[j]*mg->groups[group_name].weight[j];
  m1_ws += m1[j]*mg->groups[group_name].weight[j];
  m2_ws += m2[j]*mg->groups[group_name].weight[j];

  f1_s += f1[j];
  f2_s += f2[j];
  m1_s += m1[j];
  m2_s += m2[j];
  }

  if(remove_trio) continue;

  // make appropriate re-coding for chrX for male offspring
  if(mgd.isChrX && sex==1) f1_ws=f1_s=f2_ws=f2_s=0;

  // output CH haloptype info
  std::string group_name = mg->group_names[current_group_index];
  if(f1_s>0 && m1_s>0)
  {
   for(int j=0; j<f1.size(); j++)
   {
    if(f1[j]==0 && f2[j]==0 && m1[j]==0 && m2[j]==0 || f1[j]<0 || f2[j]<0 || m1[j]<0 || m2[j]<0) continue;
    fprintf(CH, "%s\t%s\t%s\t%s\t%d\t%d|%d\t%d|%d\n", group_name.c_str(), pheno->pids[i].c_str(), pheno->sexes[i]==1 ? "M" : "F", mg->groups[group_name].chr[j].c_str(), mg->groups[group_name].pos[j], f1[j], f2[j], m1[j],m2[j]);
    fflush(CH);
   }
  }

  double g1,g2,g3,g4, gm;
  double g[4];

  int rand_idx = rand()%4;

  // based on the above haplotype coding, various forms of gtdt can be constructed
  // 1: weighted sum
  g[0] = f1_ws + m1_ws;
  g[1] = f1_ws + m2_ws;
  g[2] = f2_ws + m1_ws;
  g[3] = f2_ws + m2_ws;
  gm = double(g[0]+g[1]+g[2]+g[3])/4.0;

  g1 = permute ? g[rand_idx] : g[0];
  tdt_ws_total += (g1 - gm);
  tdt_ws_total_var += double(pow(g[0]-gm,2)+pow(g[1]-gm,2)+pow(g[2]-gm,2)+pow(g[3]-gm,2) ) / 4.0;

  // 2: sum (equal weight)
  //the following 4 variable holds the t:nt values;
  f1_s_total += f1_s;
  f2_s_total += f2_s;
  m1_s_total += m1_s;
  m2_s_total += m2_s;

  g[0] = f1_s + m1_s;
  g[1] = f1_s + m2_s;
  g[2] = f2_s + m1_s;
  g[3] = f2_s + m2_s;
  gm = (g[0]+g[1]+g[2]+g[3])/4.0;

  g1 = permute ? g[rand_idx] : g[0];
  tdt_s_total += double(g1-gm);
  tdt_s_total_var += double(pow(g[0]-gm,2)+pow(g[1]-gm,2)+pow(g[2]-gm,2)+pow(g[3]-gm,2) ) / 4.0;

  // 3: carrier
  g[0] = f1_s+m1_s>0 ? 1.0 : 0.0;
  g[1] = f1_s+m2_s>0 ? 1.0 : 0.0;
  g[2] = f2_s+m1_s>0 ? 1.0 : 0.0;
  g[3] = f2_s+m2_s>0 ? 1.0 : 0.0;
  gm = (g[0]+g[1]+g[2]+g[3])/4.0;

  g1 = permute ? g[rand_idx] : g[0];
  tdt_c_total += double(g1 - gm);
  tdt_c_total_var += double(pow(g[0]-gm,2)+pow(g[1]-gm,2)+pow(g[2]-gm,2)+pow(g[3]-gm,2) ) / 4.0;

  // 4: compound het
  // 4.1: Indicator (CH=1 and others=0)
  g[0] = f1_s>0 && m1_s>0 ? 1.0 : 0.0;
  g[1] = f1_s>0 && m2_s>0 ? 1.0 : 0.0;
  g[2] = f2_s>0 && m1_s>0 ? 1.0 : 0.0;
  g[3] = f2_s>0 && m2_s>0 ? 1.0 : 0.0;
  gm = (g[0]+g[1]+g[2]+g[3])/4.0;

  g1 = permute ? g[rand_idx] : g[0];
  tdt_ch_total += double(g1 - gm);
  tdt_ch_total_var += double(pow(g[0]-gm,2) + pow(g[1]-gm, 2) + pow(g[2]-gm,2) + pow(g[3]-gm,2) ) / 4.0;

  // 4.2: Weighted (CH=ws and others=0)
  g[0] = f1_ws>0 && m1_ws>0 ? (f1_ws+m1_ws) : 0.0;
  g[1] = f1_ws>0 && m2_ws>0 ? (f1_ws+m2_ws) : 0.0;
  g[2] = f2_ws>0 && m1_ws>0 ? (f2_ws+m1_ws) : 0.0;
  g[3] = f2_ws>0 && m2_ws>0 ? (f2_ws+m2_ws) : 0.0;
  gm = (g[0]+g[1]+g[2]+g[3])/4.0;

  g1 = permute ? g[rand_idx] : g[0];
  tdt_ch_w_total += double(g1 - gm);
  tdt_ch_w_total_var += double(pow(g[0]-gm,2) + pow(g[1]-gm,2) + pow(g[2]-gm,2) + pow(g[3]-gm,2) ) / 4.0;

  // 6: two-allele additive model
  g[0] = f1_s>0 ? 1.0 : 0.0 + m1_s>0 ? 1.0 : 0.0;
  g[1] = f1_s>0 ? 1.0 : 0.0 + m2_s>0 ? 1.0 : 0.0;
  g[2] = f2_s>0 ? 1.0 : 0.0 + m1_s>0 ? 1.0 : 0.0;
  g[3] = f2_s>0 ? 1.0 : 0.0 + m2_s>0 ? 1.0 : 0.0;
  gm = (g[0]+g[1]+g[2]+g[3])/4.0;

  g1 = permute ? g[rand_idx] : g[0];
  tdt_2a_additive_total += double(g1 - gm);
  tdt_2a_additive_total_var += double(pow(g[0]-gm,2)+pow(g[1]-gm,2)+pow(g[2]-gm,2)+pow(g[3]-gm,2) ) / 4.0;

 } //for all individuals


 double df_double=1.0;
 double cdf;
 double t;

 // tdt statistics and p values
 // 1: weighted sum
 t = tdt_ws_total_var>0 ? pow(tdt_ws_total,2)/tdt_ws_total_var : NAN;
 double z_ws = tdt_ws_total_var>0 ? tdt_ws_total/sqrt(tdt_ws_total_var) : NAN;
 double p_ws;
 cumchi ( &t, &df_double, &cdf, &p_ws );

 // 2: sum (equal weight)
 t = tdt_s_total_var>0 ? pow(tdt_s_total,2)/tdt_s_total_var : NAN;
 double z_s = tdt_s_total_var>0 ? tdt_s_total/sqrt(tdt_s_total_var) : NAN;
 double p_s;
 cumchi ( &t, &df_double, &cdf, &p_s );

 // 3: carrier
 t = tdt_c_total_var>0 ? pow(tdt_c_total,2)/tdt_c_total_var : NAN;
 double z_c = tdt_c_total_var>0 ? tdt_c_total/sqrt(tdt_c_total_var) : NAN;
 double p_c;
 cumchi ( &t, &df_double, &cdf, &p_c );

 // 4: compound het
 // 4.1: indictor
 t = tdt_ch_total_var>0 ? pow(tdt_ch_total,2)/tdt_ch_total_var: NAN;
 double z_ch = tdt_ch_total_var>0 ? tdt_ch_total/sqrt(tdt_ch_total_var) : NAN;
 double p_ch;
 cumchi ( &t, &df_double, &cdf, &p_ch );

 // 4.2: weighted
 t = tdt_ch_w_total_var>0 ? pow(tdt_ch_w_total,2)/tdt_ch_w_total_var: NAN;
 double z_ch_w = tdt_ch_w_total_var>0 ? tdt_ch_w_total/sqrt(tdt_ch_w_total_var) : NAN;
 double p_ch_w;
 cumchi ( &t, &df_double, &cdf, &p_ch_w );

 // 6: two-allele additive model
 //save stats to an output vector
 t = tdt_2a_additive_total_var>0 ? pow(tdt_2a_additive_total,2)/tdt_2a_additive_total_var: NAN;
 double z_2a_additive = tdt_2a_additive_total_var>0 ? tdt_2a_additive_total/sqrt(tdt_2a_additive_total_var) : NAN;
 double p_2a_additive;
 cumchi ( &t, &df_double, &cdf, &p_2a_additive );

 stats[0]=p_s;
 stats[1]=z_s;
 stats[2]=p_ws;
 stats[3]=z_ws;
 stats[4]=p_c;
 stats[5]=z_c;
 stats[6]=p_ch;
 stats[7]=z_ch;
 stats[8]=p_ch_w;
 stats[9]=z_ch_w;
 stats[10] = f1_s_total;
 stats[11] = f2_s_total;
 stats[12] = m1_s_total;
 stats[13] = m2_s_total;
 stats[14] = p_2a_additive;
 stats[15] = z_2a_additive;
}

void PedDataAnalyzer::ExtractSubsetRows(Matrix &new_m, Matrix &m, std::vector<int> &index)
{
 new_m.Dimension(index.size(), m.cols);
 for(int i=0; i<index.size(); i++)
  for(int j=0; j<m.cols; j++)
   new_m[i][j] = m[index[i]][j];
}

void PedDataAnalyzer::ExtractSubsetRows(Vector &new_v, Vector &v, std::vector<int> &index)
{
 new_v.Dimension(index.size());
 for(int i=0; i<index.size(); i++)
   new_v[i] = v[index[i]];
}

