#ifndef __PEDDATAANALYZER_H__
#define __PEDDATAANALYZER_H__

#include "Pedigree.h"
#include "PedPhenoData.h"
#include "MarkerGroup.h"
#include "MarkerGroupData.h"
//#include "LinearRegression.h"
//#include "LogisticRegression.h"
#include "CmdPar.h"
//#include "Skat.h"

#include <cassert>
#include <string>
#include <set>
#include <map>
#include <vector>
#include <algorithm>
#include <iostream>
#include <fstream>
#include "Argument.h"
#include "IO.h"
#include "tabix.h"
#include "PeopleSet.h"
#include "RangeList.h"
#include "Utils.h"
#include "VCFUtil.h"

using std::ofstream;

class PedDataAnalyzer
{
 public:
  Pedigree *ped;
  PedPhenoData *pheno;
  MarkerGroup *mg;
  CmdPar *par;
  double min_freq, max_freq;
  int min_count, max_count;
  bool toEstimateFreq;
  std::vector<bool> isGenotyped;
  std::string group_name;
  int group_size;
  MarkerGroupData mgd;

  ofstream outstream; // output file handler
  FILE* CH;
  int affection_index;
  int ignore_mi_trio_cnt;

  int GT_idx;
  int DP_idx;
  int GQ_idx;
  std::vector<std::string> pids;
  std::vector<std::string> pids_with_data;
  std::map<std::string, std::pair<int, int> > pid2traverse;
  std::map<std::pair<int, int>, std::string > traverse2pid;

  int current_group_index;

 public:
  PedDataAnalyzer();
 ~PedDataAnalyzer();

  void SetPedPhenoData(PedPhenoData *phenotypes) { pheno = phenotypes; ped = pheno->ped;}
  void SetMarkerGroup(MarkerGroup *m) { mg = m; }
  void SetMinFreq(double f){min_freq = f; }
  void SetCmdPar(CmdPar *cmdpar) { par = cmdpar; /*if(par->max_count>0) par->min_count=0; if(par->max_freq>0) par->min_freq=0;*/ }
  void ExtractSubsetRows(Matrix&, Matrix &m, std::vector<int> &index);
  void ExtractSubsetRows(Vector &, Vector &v, std::vector<int> &index);
  void OpenVCF4Read(std::string);
  void MapPID2Traverse();
  void LoadAndAnalyze(std::string, std::string);
  void WrireHeader(FILE *h);
  void Collapsing_TDT();
  void CollapsingTDT_Carrier();
  void CollapsingTDT_WSS();
  void CollapsingTDT_CompoundHetero();
  void CalcTDTStats(std::vector<double>&, bool permute);
};

#endif
