#include "PedPhenoData.h"

PedPhenoData::PedPhenoData()
{
 adjustCovariates = true;
 Xnull_dicho = NULL;
 ynull_dicho = NULL;
 Xnull_quant = NULL;
 ynull_quant = NULL;
}

PedPhenoData::~PedPhenoData()
{
 if(Xnull_dicho!=NULL) delete [] Xnull_dicho;
 if(ynull_dicho!=NULL) delete [] ynull_dicho;
 if(Xnull_quant!=NULL) delete [] Xnull_quant;
 if(ynull_quant!=NULL) delete [] ynull_quant;
}

void PedPhenoData::SetPedigree(Pedigree *ptr)
{
 ped = ptr;
 nIndividuals=0;
 for(int i=0; i<ped->familyCount; i++)
   nIndividuals += ped->families[i]->count;

 if(ptr->affectionCount==0) error("No disease was found in the dat file!\n");

 Person *p;
 p = ped->persons[ped->families[0]->path[0]];

 affectionCount = p->affectionCount;
 traitCount = p->traitCount;
 covariateCount = adjustCovariates==true ? p->covariateCount : 0;

  affections.Dimension(affectionCount, nIndividuals);
  isDiagnosed.Dimension(affectionCount, nIndividuals);
  isDiagnosed.Zero();

  traits.Dimension(traitCount, nIndividuals);
  isPhenotyped.Dimension(traitCount, nIndividuals);
  isPhenotyped.Zero();

  covariates.Dimension(covariateCount, nIndividuals);
  isControlled.Dimension(covariateCount, nIndividuals);
  isControlled.Zero();

  isAllControlled.Dimension(nIndividuals);
  isAllControlled.Set(1.0);

  if(affectionCount>0) { Xnull_dicho = new Matrix[affectionCount]; ynull_dicho = new Vector[affectionCount]; }
  if(traitCount>0) { Xnull_quant = new Matrix[traitCount]; ynull_quant = new Vector[traitCount]; }
}

void PedPhenoData::GetPhenotypeData()
{
      int pid_index = 0;
      Person *p;

	for(int i=0; i<ped->familyCount; i++)
	 for(int j=0; j<ped->families[i]->count; j++)
         {
	  p = ped->persons[ped->families[i]->path[j]];

          for(int k=0; k<affectionCount; k++)
	  {
		 affections[k][pid_index] = p->affections[k]-1;
		 if(int(affections[k][pid_index])==0 || int(affections[k][pid_index])==1)
		  isDiagnosed[k][pid_index] = 1.;
	 	 else isDiagnosed[k][pid_index] = 0.;
	  }
          for(int k=0; k<traitCount; k++)
	  {
		 traits[k][pid_index] = p->traits[k];
		 if(traits[k][pid_index]!=_NAN_)
		  isPhenotyped[k][pid_index] = 1.;
	 	 else isPhenotyped[k][pid_index] = 0.;
	 }

          for(int k=0; k<covariateCount; k++)
	  {
		 covariates[k][pid_index] = p->covariates[k];
		 if(covariates[k][pid_index]!=_NAN_)
		  isControlled[k][pid_index] = 1.;
	 	 else isControlled[k][pid_index] = 0.;
	
		 if(isControlled[k][pid_index]==0.) isAllControlled[pid_index] = 0.0;
	  }
          pid_index++;
         }
}

void PedPhenoData::GetPIDs()
{
  pids.clear();
  pids_map.clear();
  isFounder.clear();
  sexes.clear();
  int counter = 0;
  std::string pid;
  nFounders = 0;

  bool founderFlag;
  int sex;

  for(int i=0; i<ped->familyCount; i++)
    for(int j=0; j<ped->families[i]->count; j++)
     {
            Person *p;
            p = ped->persons[ped->families[i]->path[j]];
	    pid = std::string(p->pid.c_str());
	    sex = p->sex;

	    if(p->isFounder()) { pids_fa_index[pid] = -1; pids_mo_index[pid] = -1; founderFlag = true; nFounders++;}
	    else {
		pids_fa_index[pid] = pids_map[p->father->pid.c_str()];
		pids_mo_index[pid] = pids_map[p->mother->pid.c_str()];
	 	founderFlag = false;
	    }

            pids.push_back(std::string(p->pid.c_str()) );
	    pids_map[std::string(p->pid.c_str())] = counter++;
	    isFounder.push_back(founderFlag);
	    sexes.push_back(sex);
     }

  pseudo_freq = 1.0/double(nFounders*2);
}

// Get the NULL data matrices for all traits
void PedPhenoData::GetNullData()
{
 if(comm_idx_mat_dicho.size()==0 && comm_idx_mat_quant.size()==0) GetCommIndex(); 

 std::vector<int> comm_idx;

 for(int i=0; i<affectionCount; i++)
 {
  comm_idx = comm_idx_mat_dicho[i];
  Xnull_dicho[i].Dimension(comm_idx.size(), covariates.rows+1);
  ynull_dicho[i].Dimension(comm_idx.size());

  for(int j=0; j<Xnull_dicho[i].rows; j++)
  { 
   for(int k=0; k<Xnull_dicho[i].cols; k++)
    Xnull_dicho[i][j][k] = k==0 ? 1.0 : covariates[k-1][comm_idx[j]];
   ynull_dicho[i][j] = affections[i][comm_idx[j]];
  }
 }

 for(int i=0; i<traitCount; i++)
 {
  comm_idx = comm_idx_mat_quant[i];
  Xnull_quant[i].Dimension(comm_idx.size(), covariates.rows+1);
  ynull_quant[i].Dimension(comm_idx.size());

  for(int j=0; j<Xnull_quant[i].rows; j++)
  { 
   for(int k=0; k<Xnull_quant[i].cols; k++)
    Xnull_quant[i][j][k] = k==0 ? 1.0 : covariates[k-1][comm_idx[j]];
   ynull_quant[i][j] = traits[i][comm_idx[j]];
  }
 }
}

void PedPhenoData::GetCommIndex()
{
 comm_idx_mat_dicho.clear();
 comm_idx_mat_quant.clear();

 if(affectionCount>0)
 {
  for(int t=0; t<affections.rows; t++)
  {
   std::vector<int> comm_idx_vec;
   for(int i=0; i<isDiagnosed[t].Length(); i++)
    if(isGenotyped[i] && isDiagnosed[t][i]>0.0 && isAllControlled[i]>0.0)
	comm_idx_vec.push_back(i);
   comm_idx_mat_dicho.push_back(comm_idx_vec);
   if(comm_idx_vec.size()==0) error("No samples with all genotype, phenotype and covaiates data for #%d disease!\n", t+1);
   }
 }

 if(traitCount>0)
 {
   for(int t=0; t<traits.rows; t++)
   {
    std::vector<int> comm_idx_vec;
    for(int i=0; i<isPhenotyped[t].Length(); i++)
     if(isGenotyped[i] && isPhenotyped[t][i]>0.0 && isAllControlled[i]>0.0)
	comm_idx_vec.push_back(i);
    comm_idx_mat_quant.push_back(comm_idx_vec);
   if(comm_idx_vec.size()==0) error("No samples with all genotype, phenotype and covaiates data for the #%d quantitative trait!\n", t+1);
   }
 }
}
