#ifndef __PEDPHENODATA_H__
#define __PEDPHENODATA_H__

#include "Pedigree.h"
#include <string>
#include <vector>
#include <map>
#include "MathVector.h"
#include "MathMatrix.h"

using namespace std;

class PedPhenoData
{
 public:
  Pedigree *ped;
  std::vector<std::string> pids;
  std::map<std::string, int> pids_map;
  std::map<std::string, int> pids_fa_index;
  std::map<std::string, int> pids_mo_index;
  std::vector<std::string> affectionNames;
  std::vector<bool> isFounder;
  std::vector<int> sexes;

  int nIndividuals;
  int nFounders;
  double pseudo_freq;

  std::vector<bool> isGenotyped;
  int affectionCount;
  Matrix affections;
  Matrix isDiagnosed;

  int traitCount;
  Matrix traits;
  Matrix isPhenotyped;

  int covariateCount;
  Matrix covariates;
  Matrix covariates_with_one;
  Matrix isControlled;
  Vector isAllControlled;
  bool adjustCovariates;

  Matrix *Xnull_dicho;
  Vector *ynull_dicho;
  Matrix *Xnull_quant;
  Vector *ynull_quant;

  std::vector<std::vector<int> > comm_idx_mat_dicho;
  std::vector<std::vector<int> > comm_idx_mat_quant;

 public:
  PedPhenoData();
 ~PedPhenoData();
  void SetPedigree(Pedigree *p);
  void SetAdjustCovariates(bool flag) { adjustCovariates = flag; }
  void SetIsGenotyped(std::vector<bool>& v) { isGenotyped = v;}
  void GetPIDs();
  void GetPhenotypeData();
  void GetNullData();
  void GetCommIndex();
};

#endif
