#include "MarkerGroup.h"
#include <iostream>
#include "Parameters.h"
#include "Pedigree.h"
#include "PedDataAnalyzer.h"
#include "CmdPar.h"

using namespace std;

int main(int argc, char *argv[])
{
 String pedFile, datFile, markerFile, vcfFile, outFile;
 String chr_range, range_list_file;
 String disease;

 int nthreads=1;
 bool sum, wss, collapsing, comphet, comphet_weight, all_tests;
 sum=wss=collapsing=comphet=comphet_weight=false;
 all_tests = true;

 double min_freq = 0.0;
 double max_freq = 0.01;
 double min_avg_depth = 0.0;
 double max_avg_depth = 0.0;

 int min_count = -1;
 int max_count = -1;
 int min_dp = 0;
 double min_gq = 0.0;

 bool pass = false;
 double min_qual = 0;
 double max_missing_rate = 1.0;
 bool missing_as_ref = false;
 bool phased = false;
 bool ignore_mi_trio = false;

 String chrX = "X";
 String chrY = "Y";
 String chrMT = "MT";

 int permute = 0;
 int seed = 13579;

 ParameterList pl;

 BEGIN_LONG_PARAMETERS(longParameters)
  LONG_PARAMETER_GROUP("Groupwise test")
    LONG_PARAMETER("sum", &sum)
    LONG_PARAMETER("wss", &wss)
    LONG_PARAMETER("cmc", &collapsing)
    LONG_PARAMETER("comphet", &comphet)
    LONG_PARAMETER("comphet-weight", &comphet_weight)
    LONG_PARAMETER("all", &all_tests)
  LONG_PARAMETER_GROUP("Variant filters")
    LONG_DOUBLEPARAMETER("min_qual", &min_qual)
    LONG_DOUBLEPARAMETER("min_maf", &min_freq)
    LONG_DOUBLEPARAMETER("max_maf", &max_freq)
    LONG_DOUBLEPARAMETER("min_avg_dp", &min_avg_depth)
    LONG_DOUBLEPARAMETER("max_avg_dp", &max_avg_depth)
    LONG_DOUBLEPARAMETER("max_missing_rate", &max_missing_rate)
    LONG_PARAMETER("pass", &pass)
  LONG_PARAMETER_GROUP("Genotype filters")
    LONG_INTPARAMETER("min_dp", &min_dp)
    LONG_DOUBLEPARAMETER("min_gq", &min_gq)
    LONG_PARAMETER("missing_as_ref", &missing_as_ref)
    //LONG_PARAMETER("ignore_noninformative", &ignore_noninformative)
    //LONG_INTPARAMETER("min_count", &min_count)
    //LONG_INTPARAMETER("max_count", &max_count)
  LONG_PARAMETER_GROUP("Phased genotypes")
    LONG_PARAMETER("phased", &phased)
  LONG_PARAMETER_GROUP("Mendelian inconsistency")
    LONG_PARAMETER("ignore_mi_trio", &ignore_mi_trio)
  LONG_PARAMETER_GROUP("Disease to analyze")
    LONG_STRINGPARAMETER("disease", &disease)
  LONG_PARAMETER_GROUP("Non-autosomes")
    LONG_STRINGPARAMETER("chrX", &chrX)
    LONG_STRINGPARAMETER("chrY", &chrY)
    LONG_STRINGPARAMETER("MT", &chrMT)
  LONG_PARAMETER_GROUP("Empirical p-values")
    LONG_INTPARAMETER("permute", &permute)
    LONG_INTPARAMETER("seed", &seed)
  LONG_PARAMETER_GROUP("Multi-threading")
    LONG_INTPARAMETER("nthreads", &nthreads)
  END_LONG_PARAMETERS();

  pl.Add(new StringParameter('p', "pedfile", pedFile));
  pl.Add(new StringParameter('d', "datfile", datFile));
  pl.Add(new StringParameter('g', "groupfile", markerFile));
  pl.Add(new StringParameter('v', "vcffile", vcfFile));
  pl.Add(new StringParameter('o', "outfile", outFile));
  pl.Add(new LongParameters("Additional Options", longParameters));

  pl.Read(argc, argv);

  if(sum || wss || collapsing || comphet || comphet_weight) all_tests = false;

  pl.Status();

  if(min_freq > max_freq || min_count>max_count) error("min freq or count is greater than max freq or count.\n");


 srand(seed);
// srand (time(NULL));


  CmdPar par;
  par.phased = phased;
  par.sum = sum;
  par.wss = wss;
  par.collapsing = collapsing;
  par.comphet = comphet;
  par.comphet_weight = comphet_weight;
  par.all = all_tests;
  par.min_freq = min_freq;
  par.max_freq = max_freq;
  par.min_count = min_count;
  par.max_count = max_count;
  par.min_avg_depth = min_avg_depth;
  par.max_avg_depth = max_avg_depth;
  par.min_dp = min_dp;
  par.min_gq = min_gq;
  par.pass = pass;
  par.missing_as_ref = missing_as_ref;
  par.max_missing_rate = max_missing_rate;
  par.min_qual = min_qual;
  par.disease = disease;
  par.chrX = std::string(chrX.c_str());
  par.chrY = std::string(chrY.c_str());
  par.chrMT = std::string(chrMT.c_str());
  par.permute = permute;
  par.ignore_mi_trio = ignore_mi_trio;

  if(pedFile.Length()==0) error("ped file not provided [-p]\n");
  if(datFile.Length()==0) error("dat file not provided [-d]\n");
  if(markerFile.Length()==0) error("group file not provided [-g]\n");
  if(vcfFile.Length()==0) error("vcf file not provided [-v]\n");
  if(outFile.Length()==0) error("output file not provided [-o]\n");


  IFILE datFH = ifopen(datFile, "r");
  IFILE pedFH = ifopen(pedFile, "r");

  if(datFH==NULL) error("Open dat file failed!\n");
  if(pedFH==NULL) error("Open ped file failed!\n");

  Pedigree ped;
  ped.Prepare(datFH);
  ped.Load(pedFH);

  PedPhenoData pheno;
  pheno.SetPedigree(&ped);
  pheno.GetPhenotypeData();

  PedKinship pedkin;
  pedkin.SetPedKinship(&ped);

  MarkerGroup mg;
  mg.chrX = par.chrX;
  mg.chrY = par.chrY;
  mg.chrMT = par.chrMT;
  if(min_freq>0.0 && max_freq<1.0) mg.SetMinMaxFreq(min_freq, max_freq);
  mg.LoadMarkerGroupInfo(std::string(markerFile.c_str()));
//  cout<<mg.group_names.size()<<endl;
 //mg.PrintGroupInfo();

 PedDataAnalyzer analyzer;
 analyzer.SetCmdPar(&par);
 analyzer.SetPedPhenoData(&pheno);
 analyzer.SetMarkerGroup(&mg);
 analyzer.LoadAndAnalyze(std::string(vcfFile.c_str()), std::string(outFile.c_str()));

}
