#include "MarkerGroup.h"
#include <iostream>
#include "Parameters.h"
#include "Pedigree.h"
#include "PedDataAnalyzer.h"
#include "CmdPar.h"
#include "Skat.h"

using namespace std;

int main(int argc, char *argv[])
{
 String pedFile, datFile, markerFile, vcfFile, outFile;
 String chr_range, range_list_file;
 int nthreads=1;
 bool score, wald, lrt;
 String collapsing;
 double min_freq=1.0;
 score = wald = lrt = false;

 ParameterList pl;

 BEGIN_LONG_PARAMETERS(longParameters)
  LONG_PARAMETER_GROUP("Groupwise test")
    LONG_STRINGPARAMETER("collapsing", &collapsing)
    LONG_DOUBLEPARAMETER("minfreq", &min_freq)
    LONG_PARAMETER("score", &score)
    LONG_PARAMETER("wald", &wald)
    LONG_PARAMETER("lrt", &lrt)
  LONG_PARAMETER_GROUP("Multi-threading")
    LONG_INTPARAMETER("nthreads", &nthreads)
  END_LONG_PARAMETERS();

  pl.Add(new StringParameter('p', "pedfile", pedFile));
  pl.Add(new StringParameter('d', "datfile", datFile));
  pl.Add(new StringParameter('m', "vgcfile", markerFile));
  pl.Add(new StringParameter('v', "vcffile", vcfFile));
  pl.Add(new StringParameter('o', "outfile", outFile));
  pl.Add(new LongParameters("Additional Options", longParameters));

  pl.Read(argc, argv);

  pl.Status();

  CmdPar par;
  par.score = score;
  par.wald = wald;
  par.lrt = lrt;
  par.collapsing = std::string(collapsing.c_str());
  
  IFILE datFH = ifopen(datFile, "r");
  IFILE pedFH = ifopen(pedFile, "r");
  
  Pedigree ped;
  ped.Prepare(datFH);
  ped.Load(pedFH);

  PedPhenoData pheno;
  pheno.SetPedigree(&ped);
  pheno.GetPhenotypeData();

  PedKinship pedkin;
  pedkin.SetPedKinship(&ped);

  MarkerGroup mg;
  if(min_freq<1.0) mg.SetMinFreq(min_freq);
  mg.LoadMarkerGroupInfo(std::string(markerFile.c_str()));
  cout<<mg.group_names.size()<<endl;
 //mg.PrintGroupInfo();

 PedDataAnalyzer analyzer;
 analyzer.SetPedPhenoData(&pheno);
 analyzer.SetMarkerGroup(&mg);
 analyzer.SetCmdPar(&par);
 analyzer.LoadAndAnalyze(std::string(vcfFile.c_str()), std::string(outFile.c_str()));

 Eigen::MatrixXf a, b;
 Skat s;
 s.CalculatePValue(a, a, 10.0);

}
